function h = viewMultinorm(Sys, type, in1, in2)
% VIEWMULTINORM Displays the quadratic, sos or polytope multinorm.
%
%   S.VIEWMULTINORM('quad', Q) displays the multinorm built on system S where
%   Q is a cell arrays of matrices describing the quadratic norm at each
%   node.
%   S.VIEWMULTINORM('sos', Q, d) displays the multinorm built on the sos-lifted
%   system S where Q contains the (lifted) quadratic forms
%   The Q should come from the output of the cjsrQuadMultinorm method and have
%   the appropriate dimensions.
%   S.VIEWMULTINORM('poly', P) displays the complex-polytope multinorm coming
%   from the output of cjsrCplxPolytopeMultinorm.
%   H = S.VIEWMULTINORM(...) also returns the handle H to the figure.
%
%   See also VIEW, CJSRQUADMULTINORM, CJSRCPLXPOLYTOPEMULTINORM.

quad = strcmp(type, 'quad') ;
sos  = strcmp(type, 'sos') ;
poly = strcmp(type, 'poly') ;

if nargin <= 2
    error('Needs at least 3 input arguments') ;
end
if ~ ischar(type)
    error('Type should be a string') ;
end

if ~ (quad || sos || poly)
    error('Type should be quad, sos or poly')
end
if quad
    Q = in1 ;
    d = 1 ;
    if nargin ~= 3
        error('With quad needs 3 arguments') ;
    end
    if ~ iscell(Q) || numel(Q) ~= Sys.getNNodes()
        error('Q should be a cell array with as many entries as the number of nodes') ;
    end
elseif sos
    Q = in1 ;
    d = in2 ;
    if nargin ~= 4
        error('With sos needs 4 arguments') ;
    end
    if ~ iscell(Q) || numel(Q) ~= Sys.getNNodes()
        error('Q should be a cell array with as many entries as the number of nodes') ;
    end
    if d <= 0 || ~ isscalar(d)
        error('d needs to be a >= 1 scalar') ;
    end
elseif poly
    P = in1 ;
    d = 1 ;
    if nargin ~= 3
        error('With poly needs 3 arguments') ;
    end
    if ~ iscell(P) || numel(P) ~= Sys.getNNodes()
        error('P should be a cell array with as many entries as the number of nodes') ;
    end
else
    error('Wrong type') ;
end

h = figure ;
nNodes = Sys.getNNodes() ;
n = ceil(sqrt(nNodes)) ;
m = ceil(nNodes/n) ;
if quad || sos
    dim = size(Q{1},1) ;
else
    dim = size(P{1},1) ;
end

% Preprocess
if poly
    nn = 100 ;
elseif quad || sos
    nn = 250 ;
end
theta = linspace(0, 2*pi, nn) ;
r = ones(1, nn) ;
x = r.*cos(theta) ;
y = r.*sin(theta) ;

if quad || sos
    [~, monomials] = sosLiftMatrix(eye(2), d) ;
    N = size(monomials, 1) ;
    Xd = zeros(N, nn) ;
    for i = 1:N
        coef = factorial(d) / factorial(monomials(i, 1)) / factorial(monomials(i, 2)) ;
        Xd(i, :) = sqrt(coef) * x.^(monomials(i, 1)) .* y.^(monomials(i, 2)) ;
    end
elseif poly
    X = [x ; y] ;
end

opts = cjsrSettings() ;
for node = 1:nNodes
    subplot(m, n, node) ;
    if nchoosek(2+d-1,d) == dim
        if quad || sos
            Qq = Q{node} ;
            rnorm = zeros(1, nn) ;
            for i = 1:nn
                rnorm(i) = sqrt(Xd(:,i)' * Qq * Xd(:,i))^(1/d) ;
            end
        elseif poly
            Pp = P{node} ;
            % rnorm = polyNorm(Pp, X) ;
            for i = 1:nn
                rnormi = polyNorm(Pp, X(:,i), opts) ;
                if isempty(rnormi) % Rank-deficient polytope
                    error('Cannot plot rank-deficient polytope') ;
                end
                rnorm(i) = rnormi ;
            end
        end        
        runit = r ./ rnorm ;
        polar(theta, runit) ;
        title(sprintf('Node %d', node)) ;
    else
        error('The matrices should be 2x2 or come from a 2x2 lifted matrix.') ;
    end
    axis equal ;
end
end