%% NOTES:

% This demo relies on the CSSystem toolbox (see https://www.mathworks.com/matlabcentral/fileexchange/52723-the-cssystem-toolbox). 
% A working version of the toolbox is packaged with the demo.
% Please see the file: CSSystemV1/Readme.txt for installation information.
% For all the main utilities, 
% it requires the installation of a solver such
% as Sedumi or SDPT3. 
% Note from Matt [12/10/2016]:
% on 12/10/2016, in case you don't have the solvers installed,
% you may just uncomment the following line. 

% urlwrite('http://www.tbxmanager.com/tbxmanager.m', 'tbxmanager.m'); tbxmanager; savepath;  tbxmanager install sedumi;

% This should download and install sedumi automatically.(thanks to www.tbxmanager.com).
addpath(genpath('CSSystemV1')); % Codes for handling linear switching systems 
% defined on graphs. We are going to focus on
% path-complete graphs.
addpath(genpath('utilities')); % Tools allowing to construct observer graphs, label nodes for representation, etc...

close all % We are going to be generating figures.
clc
fprintf('_____DEMO______\n');
fprintf(' Path-Complete Graphs and Common Lyapunov functions\n');
fprintf(' Section 5 \n');
fprintf('_________________ \n\n')
%% Generating the matrices.
A = [0.97, 0.58; 0.17, 0.50];
B = [0 ; 1];
K = [-0.55, 0.24];
ABK = A+B*K;
fprintf('Matrices: \n')
A
B
K
%ABK = A+B*K;
fprintf('Given M >= 1, we consider the system x(t+1) = H_{s(t)} x(t)\n')
fprintf('where H_s is one of the M matrices H_s = A^(s-1)ABK^(M-s+1), s = 1...M \n')
%% The case M = 1.
M = 1;
spectral_radiusM1 = max(abs(eigs(ABK)));
message = sprintf('___\n [M = 1],\n System is STABLE, \n Proof: Spectral radius of H_1 = A+BK =  %1.2d < 1 \n', spectral_radiusM1^(1/M));
fprintf(message)

%% The case M = 2
% Generating the matrices. Note that \tilde{A}_1 is a function of M itself.
% Thus, we need to generate them for every M.
M = 2;
matM2 = cell(M,1);
for k = 1 : M
    matM2{k} = (A^(k-1))*(ABK^(M-k+1));
end
% For M = 2, we have two modes.
% We are dealing with a switching system already :/
% Stability analysis becomes more complicated than just checking the stability of both matrices.
% BTW, they are both stable.

% ABOUT CHECKING FOR THE EXISTENCE OF A PCLF.
% In order to understand the code below, we need to explain a little bit
% the techniques discussed in Philippe et al., Stability of discrete-time switching systems with constrained
% switching sequences, Automatica 72 - 2016. 
% A constrained switching system is a discrete-time switching system where 
% the switching system is constrained to corresponds to paths in a labeled
% graph. Using a path-complete graph, the any admissible sequence is
% accepted.
% We define a graph by its edges [node source, node dest, label].
G8a = [1 1 1; % node 1 to node 1 with label 1
    1 2 1; % node 1 to node 2 with label 2
    2 2 2; % node 2 to node 2 with label 2
    2 1 2]; % node 2 to node 1 with label 2
% We then define the system by the pair Graph + Matrices.
Sys_G8a = CSSystem(G8a, matM2);

% This structure being defined, we check wether or not it has a Lyapunov
% function of the form (i,j,s) is an edge in the graph => (V_j(A_s x ) <=
% V_i(x)).
opt = cjsrSettings('verbose',0);
[eM2,QM2] = Sys_G8a.cjsrQuadMultinorm(opt);

% If the first output of this function eM2 is <=1, then the answer is yes,
% there is a Lyapunov function. 
% The link with path-complete Lyapunov functions is that if we use a PC
% graph to define the constrained switching system, the Lyapunov function is a PCLF for the
% arbitrary switching system.
message = sprintf('___\n [M = 2],\n System is STABLE, \n Proof: Path-Complete Lyapunov function for the graph G8a = (S,E) with: \n');
fprintf(message)
fprintf(printSystemGraph(Sys_G8a));
message = sprintf('(Exponential decay rate is < %1.2d < 1 ) \n', spectral_radiusM1^(1/M)');
fprintf(message)
%% The case M = 3
% 
% building the matrices
M = 3;
matM3 = cell(M,1);
for k = 1 : M
    matM3{k} = (A^(k-1))*(ABK^(M-k+1));
end

% We are going to consider 3 different graphs:
% the one of Figure 10a; the one of Figure 10b, and the one of Figure 11.

G10a = [
    1 1 1; 
    1 2 1; 
    1 1 2; 
    2 1 3; 
    2 2 2; 
    2 3 3; 
    3 2 2; 
    3 3 3; 
    3 1 3
    ];

G10b = [ 
    1 1 1; 
    1 1 2;
    1 2 1;
    1 3 1;
    2 1 3;
    2 2 2;
    2 3 2;
    3 4 3;
    4 1 3;
    4 2 2;
    4 3 2;
    4 4 3
    ];

G11 = [ 
    1 1 1; 
    1 2 2;
    1 3 1;
    2 1 1;
    2 2 2;
    2 3 1;
    3 2 3;
    3 3 2;
    3 4 3;
    4 2 3;
    4 3 2;
    4 4 3
    ];

Sys_G10a = CSSystem(G10a,matM3);

% Testing if PCLF with quadratic pieces for the first graph:
[e10a, Q10a] = Sys_G10a.cjsrQuadMultinorm(opt);

Sys_G10b = CSSystem(G10b,matM3);
% Testing if PCLF with quadratic pieces for the second graph:
[e10b, Q10b] = Sys_G10b.cjsrQuadMultinorm(opt);

Sys_G11 = CSSystem(G11,matM3);
% Testing if PCLF with quadratic pieces for the third graph:

[e11, Q11] = Sys_G11.cjsrQuadMultinorm(opt);


message = sprintf('___\n [M = 3],\n Tentative proof 1 (false): No Path-Complete Lyapunov function for the graph G10a = (S,E) with: \n');
fprintf(message)
fprintf(printSystemGraph(Sys_G10a));
message = sprintf(' Tentative proof 2 (false): No Path-Complete Lyapunov function for the graph G10b = (S,E) with: \n');
fprintf(message)
% This one has special node names for the example :-)
nodeNames10b = {'a', 'b1', 'b2', 'c'};
fprintf(printSystemGraph(Sys_G10b,nodeNames10b));
message = sprintf(' System is STABLE, \n Proof: Path-Complete Lyapunov function for the graph G10c = (S,E) with: \n');
fprintf(message)
[txt, nodeNames] = printSystemGraph(Sys_G11);
fprintf(txt);

%% We can now extract a CLF for the system. First we build its observer. 
[Sys_OBS_G11, nodeSets] = observerGraph(Sys_G11);
observerNames = getObserverNodeNames(nodeSets, nodeNames);
% Printing the observer.
message = sprintf( 'Constructing CLF from Observer. Observer given by: \n');
fprintf( message ); % I do realize the combo sprint/fprint is quite unnecessary.
                   % Moving on...
fprintf( printSystemGraph(Sys_OBS_G11,observerNames) );
fprintf( ' The strongly connected and complete component is on the nodes \n  ac; bc; bd.\n ')
fprintf( ' Plotting the level sets of the CLF (min (max(V_a,V_c),max(V_b,V_c), max(V_b,V_d))). \n')

%% Now we will plot the level sets :-) for M = 3.
% From the observer, we see that the sets {1,3} {2,3} {2,4} are the important ones.
M = 3;
tt = linspace(0,2*pi,1000);
xy = [cos(tt);sin(tt)];
L = cell(M,1);
Blv = cell(M,1);
figure;
hold on
for i = 1 : 4
    F{i} = @(x) sum(x'*Q11{i}.*x',2);
    L{i} = F{i}(xy);
    Blv{i} = 1./sqrt(L{i})';
    plot(xy(1,:).*Blv{i}, xy(2,:).*Blv{i},'color', [150,150,150]/255)
end
% we need to matlabize: min (A,B,C) = min(A,min(B,C))
FTot = @(x) min(...
    max(F{1}(x), F{3}(x)),...
    min(max(F{2}(x), F{3}(x)),...
         max(F{2}(x), F{4}(x))));
 BTot = 1./sqrt(FTot(xy))';
 plot(xy(1,:).*BTot, xy(2,:).*BTot,'r','linewidth',2);
axis( [-0.8, 0.8, -0.8, 0.8] )
title( 'Level sets of a CLF for the case M = 3')
 
 
%% The case M = 4
M = 4;
matM4 = cell(M,1);
for k = 1 : M
    matM4{k} = (A^(k-1))*(ABK^(M-k+1));
end
spectral_radiusM4 = max(abs(eigs(matM4{4})));

message =  sprintf('___\n [M = 4],\n System is UNSTABLE: Proof: 4th mode is given by the matrix A^3(A+BK):\n');
fprintf(message)
A^3*(A+B*K)
message =  sprintf('with a spectral radius of %1.2d > 1 \n', max(abs(eigs(A^3*(A+B*K)))));
fprintf(message)

%%
fprintf('___________________\n')
fprintf('___ END OF DEMO ___\n')
fprintf('___________________\n')
