function [ observerSystem, table ] = observerGraph( sys )
%% Construct the observer graph for the graph defining the system SYS.

 S = sys.getNNodes;
 table = baseCounter(2,S); % The sets are represented as binary vectors (1xS)
 % Base counters generates all such vectors from 0 to 2^S-1.
 table = table(2:end,end:-1:1); % Remove 0, and flip the array (leftmost entry is for node 1, etc...)
 
 numGroups = size(table,1); % Maximum nunber of nodes in the observer 
 newEdges = []; % We are going to add edges.
 
 labels = cell(1,1); % to be extended, label list of the observer.
 
 % Now, we begin with the full set, and iterate until no new nodes are found.
 [formattedEdges,labList,kLabs] = formatEdges(sys); % See the code, formatting purposes.

 binVal = (2 .^(0:S-1))'; % Assigns a number per node of the observer.
 
 visitedNodes = zeros(numGroups, 1); % Is the set of nodes already visited?
 % mark as +1 if edges have already been generated from there :-).
 
 currentSets = zeros(numGroups, 1);
 currentSets(end) = 1; % sets to explore. The last set in the table is S.
 
 edgesAdded = 0; 
 exploring = true;
 
 
 while exploring
     
     newlyExplored = zeros(numGroups,1); % The set X in the construction.
     
     setIndexes = find(currentSets); 
     % while in current set, I keep adding edges 
     % INVARIANT: If a set is in current set, no edges have been generated from it yet.
     
     for lab = 1 : kLabs % We go through all modes of the system.
         
         eLab = formattedEdges( formattedEdges(:,3) == lab,:); 
         % The edges for this label.
         
         for setID = 1 : length(setIndexes) % Get the first set from which we build an edge.
         
             set = find(table(setIndexes(setID),:) == 1); % Its this one
             destinationSet = zeros(1,S); % Where does the edge go?
             
             for v = 1 : length(set)
                 % Basically, we construct Q = \cup_s \in S \{d: (s,d,sigma) \in E\}
                 eSet = eLab(eLab(:,1) == set(v),:); 
                 destinationSet(eSet(:,2)) = 1; % We just mark a node if it is visited.
             end
             
             destSetID = destinationSet * binVal; % To known which set it was.
             
             if destSetID == 0 % Did not found anything. (Should never happen in a PC graph)
                 continue
             end             
             newlyExplored(destSetID) = 1; 
             % We found it and report our discovering to National geoGRAPHics 
             % (is there a prize for lame puns?)
             edgesAdded = edgesAdded + 1; %             
             newEdge = [setIndexes(setID), destSetID];             
             newEdges = [newEdges; newEdge];            
             labels{edgesAdded} = labList{lab}; % This is mainly due to a choice of format for the CSSystem data structure.
         end
     end    
     visitedNodes(currentSets == 1) = 1; % Current set is now visited.
     newlyExplored(visitedNodes == 1) = 0; % Sets that have been visited are not newly explored.
     currentSets = newlyExplored; % We need then to create edges from the newly explored sets.
     exploring =( sum(currentSets) > 0 ); % ... We stop if there is nothing to do left.
 end


 
% Now we need to filter the result (for the format):

removalMap = (1:numGroups)';
nodesToRemove = setdiff(removalMap, unique(newEdges));
removalMap(nodesToRemove) = 0;
k = 0;
for v = 1 : numGroups
    if removalMap(v) > 0
        k = k + 1;
        removalMap(v) = k;
    end
end
for e = 1 : size(newEdges,1);
    newEdges(e,:) = [removalMap(newEdges(e,1)), removalMap(newEdges(e,2))];
end
table = table(find(removalMap > 0),:);

observerSystem = CSSystem(newEdges, sys.getMatrices, labels);
 
  %table = table(:,end:-1:1);
end

