close all;
% Start by adding the CCSystemV1 folder and subfolders to path.
% You should make sure you are able to run the CSSystem toolbox.
% Please refer to CSSystemV1\Readme.txt for details.
% 
disp(' ')
disp('This script reproduces the results presented in');
disp('``Stability of discrete-time switching systems with constrained switching sequences.  ');
disp('by Philippe M., Essick R., Dullerud G. E., Jungers R. M., Section 4');
disp('You will need either SeDuMi or SDPT3 installed for solving LMIs.')
disp('The script relies on the CSSystem toolbox, eclosed in the .zip file. Please add the folder (and subfolders) to the matlab path.')
disp('For more details, see ``CSSystemV1\Readme.txt')
disp('Please make sure that all required files are added to the path now (solver + CSSystem).')
pause()
disp(' ')
disp('In this script, we first construct the system. Then, we compute the T-product lifts and M-path-dependent lifts.')
disp('We then approximate the CJSR using the lifts, and we finally compute the exact value of the CJSR of the system.')
disp('The script needs approximatively 60 minutes to run.')
disp('More explanations are found in the code.')
disp('Press any key to continue.')
disp(' ')
pause()

%% -------- SECTION 4 - SYSTEM'S DESCRIPTION -----------------------------
disp('... Building the system ...')
% We now fully define the system's parameters.
n = 2; % dimension
A = [0.94 0.56 ; 0.14 0.46] ; % state
B = [0 ; 1] ; % input to state
K1 = [-0.49 0.27] ; % control mode 1
K2 = [0 K1(2)] ; % control mode 2
K3 = [K1(1) 0] ; % control mode 3
K4 = [0 0] ; % control mode 4
matrices = {A+B*K1 ; A+B*K2 ; A+B*K3 ; A+B*K4} ; % matrix set \Sigma, 4 labels
edges = [1 3 1 ;
         1 2 3 ;
         2 1 2 ;
         2 3 1 ;
         3 1 2 ;
         3 2 3 ;
         3 4 4 ;
         3 3 1 ;
         4 3 1 ] ; % automaton description, each edge has the format (src, dst, label)
S = CSSystem(edges, matrices) ; % Our system.
disp('... Done ...')
disp(' ')
%% -------- SECTION 4 - Computing the lifts  -----------------------------
disp('... Computing the lifts ...')
% This example focuses on comparing two lifts: 
% - T-product lift,
% - M-path-dependent lift.
% Others can be compared - all are available within the toolbox. We will
% add pointers at the end of the test.
% ---------- Parameter selections ----------------------------------------
% We wish to obtain CJSR upper bounds which are at most (1+r)*CJSR, 
% for r = 
r = 2.^(1./(2*(1:7))) - 1+0.001; % POINTER:-> Change here for other values of r > 0.
% this gives us a range from ~41.42% to ~5.08%
% We will use the T-Lift (Subection 3.1) and the M-path-dependent Lift (Subsection 3.3). 
% Using Corollary 3.3, we compute the associated "T" and "M" (= T-1, see Thm 3.6).
T = ceil(log(n) ./ ( 2 * log(1+r) )); % POINTER :->  change here for other values for T and M.
M = T-1;
N = length(T); % How many different lifts should we compute?
%--------- STEP 1) Computing the lifts -----------------------------------
% We use cells of CSSystems objects to store the lifts.
STProdCell = cell(N,1);
SMPDepCell = cell(N,1);
% We will compare the sizes of the obtained systems (graphs).
nNodes = zeros(N,2); %1st row for TProd, 2nd row for MPathDep
nEdges = zeros(N,2);
for h = 1 : N % We compute each lift for each value of T (M = T-1)
    STProdCell{h} = S.TProductLift(T(h)); % Compute the T-Prod with parameter T(h)
    nNodes(h,1) = STProdCell{h}.getNNodes; % Extract amount of nodes
    nEdges(h,1) = STProdCell{h}.getNEdges; % Extract amount of edges
    SMPDepCell{h} = S.MDependantLift(M(h));  % Compute the M-Path-Dep with parameter T(h)
    nNodes(h,2) = SMPDepCell{h}.getNNodes; % Extract amount of nodes
    nEdges(h,2) = SMPDepCell{h}.getNEdges; % Extract amount of edges
end

%% Plotting step 1.
% Matlab Generated Figures, using Matlab 2015a.
figure1 = figure;
axes1 = axes('Parent',figure1,'YGrid','on','XGrid','on','YMinorTick','on',...
    'YScale','log',...
    'YTick',unique([nNodes, nEdges]),...
    'XTick', 1:N, ...
    'FontSize',16,...
    'Position',[0.131043841336117 0.11 0.775 0.815]);
box(axes1,'on');
hold(axes1,'on');
plot(1:N, nNodes(:,1),'Parent', axes1, 'DisplayName','T-Product Nodes','MarkerSize',30,...
    'Marker','.',...
    'Color',[0.87058824300766 0.490196079015732 0],'LineWidth',2);
plot(1:N, nEdges(:,1),'Parent', axes1,'DisplayName','T-Product Edges','MarkerSize',30,...
    'Marker','.',...
    'LineStyle','-.',...
    'Color',[0.87058824300766 0.490196079015732 0],'LineWidth',2);
plot(1:N, nNodes(:,2),'Parent', axes1,'DisplayName','M-Path Nodes','MarkerSize',10,'Marker','o',...
    'Color',[0.466666668653488 0.674509823322296 0.18823529779911],'LineWidth',2);
plot(1:N, nEdges(:,2),'Parent', axes1,'DisplayName','M-Path Edges','MarkerSize',10,'Marker','o',...
    'LineStyle','--',...
    'Color',[0.466666668653488 0.674509823322296 0.18823529779911],'LineWidth',2);
legend1 = legend(axes1,'show');
set(legend1,...
    'Position',[0.627522620342007 0.327252631888593 0.248434232334006 0.217799883313176], ...
    'EdgeColor',[1 1 1],...
    'FontSize', 12);
annotation(figure1,'textbox',...
    [0.133611691022955 0.83150414746911 0.333507297261149 0.067993364968703],...
    'String',{'Amount of Nodes or Edges'},...
    'LineStyle','none',...
    'FontSize',14,...
    'FitBoxToText','off');
annotation(figure1,'textbox',...
    [0.854906054279736 0.115086237021347 0.0474947796889537 0.0679933649687032],...
    'String',{'T'},...
    'LineStyle','none',...
    'FontSize',14);
% This ends the part of the script where we actually compute the lifts,
% which are switching systems. So far, we did not attempt any estimations
% of the CJSR.
disp('... Done ...')
pause(3)
disp('')
%% -------- SECTION 4 - CJSR'S Approximation   ----------------------------
disp('... Approximation of the CJSR (!! takes some time !!!) ... ')
% Let us set up some parameters for solving the quasi-convex optimization
% programs. 
% Our current implementation for solving the algorithm of theorem 3.1
% uses a bisection algorithm to retrieve the estimation, solving a set of
% LMIs at each steps (see Theorem 3.1).
% NOTE: we are able to detect the size of the products associated to each
% edges. When running the estimation procedure on a T-lift, the algorithm
% directly output an estimate for the original system (we provide the value
% \gamma_*^{1/T} of Theorem 3.2).
% We first set up some parameters:
    % Here we provide upper and lower bounds on the CJSR of the original system.
    % 0.9 <= rho(matrices{1}), is a lower bound on the CJSR (Lemma 3.7)
    % 1.25 >= max_i(||A_i||), is an upper bound on the CJSR (by definition
    % (1), and by submultiplicativity).
    bounds = [0.9,1.25]; % POINTER -> If you want to use different bounds.
    % Here we set up a tolerance for the bisection, 
    tol = 1e-5; % POINTER -> If you want to use another tolerance.
 opts = cjsrSettings(...
    'quad.bounds', bounds ,...
    'verbose', 0,... % POINTER -> If you don't care about seeing the bisection process, put 0 instead of 1.
    'quad.tol', tol);

% For each value of T (and M), we compare first the time it takes 
% for the algorithm to ouput an estimate (T-Product lift will be faster), 
% and the actual estimation value (M-Path-Dependent will be better).
repeatTimeMeasurement = 2; % Just to get more precise measurement, just set it to 1 for a demo.
times = zeros(N,2); % 1st row for T-Prod, 2nd row for M-Path-Dep.
estim = zeros(N,2);
disp('T-Product lift:')
for h = 1 : N % We begin with T-Prod.
    for trial = 1 : repeatTimeMeasurement
        % In order to obtain the estimation, we make a call to the
        % cjsrQuadMultinorm routine, described theoretically in Theorem
        % 3.1
        [cjsrEval, ~, nfo] = STProdCell{h}.cjsrQuadMultinorm(opts); 
        times(h,1) = times(h,1)  + 1/repeatTimeMeasurement * nfo.timeSolve; 
        % We have actually implemented time measurements within the procedure!
        % timeSolve focuses on the time taken to solve the quasi convex
        % program of Theorem 3.1 up to bisection tolerance. We discard the
        % different overhead spent in building the optimization program,
        % etc... and focus on the solving.
      
    end
   estim(h,1) = cjsrEval;    
   fprintf('T = %d, estimate = %1.3f, time required for solving = %3.3f. \n', h, estim(h,1), times(h,1))
end
%%
disp('M-Path-Dependent lift:')
for h = 1 : N % Same for M-Path-Dep
    for trial = 1 : repeatTimeMeasurement
       start = tic;
       [cjsrEval, ~, nfo] = SMPDepCell{h}.cjsrQuadMultinorm(opts);
       times(h,2) = times(h,2)  + 1/repeatTimeMeasurement * nfo.timeSolve;
    end
    estim(h,2) = cjsrEval; 
    fprintf('M = %d, estimate = %1.3f, time required for solving = %3.3f. \n', h-1, estim(h,2), times(h,2))
end

%% We now move onto the plots.
% again, the plot in the paper where obtained using Matlab 2015a.
% First, for the estimation times.
figure1 = figure;
axes1 = axes('Parent',figure1,'YMinorTick','on','YScale','log',...
    'FontSize',14);
box(axes1,'on');
hold(axes1,'on');
semilogy(T, times(:,1),'MarkerSize',30,'Marker','.',...
    'Color',[0.87058824300766 0.490196079015732 0],'LineWidth',2, 'DisplayName','TProd Execution Time');
semilogy(T, times(:,2),'MarkerSize',10,'Marker','o',...
    'Color',[0.466666668653488 0.674509823322296 0.18823529779911],'LineWidth',2, 'DisplayName','MPath Execution Time');
legend1 = legend(axes1,'show');
set(legend1,...
    'Position',[0.480654767625741 0.237301590423735 0.407142847989287 0.119047615925471], ...
    'EdgeColor',[1 1 1],...
    'FontSize', 12);
annotation(figure1,'textbox',...
    [0.151785714285713 0.826190477396762 0.14821428182934 0.0642857130794299],...
    'String',{'Time (sec)'}, ...
       'LineStyle','none',...
    'FontSize',14,...
    'FitBoxToText','off');
annotation(figure1,'textbox',...
    [0.824999999999999 0.140476191682475 0.051785713274564 0.0642857130794298],...
    'String',{'T'}, ...
       'LineStyle','none',...
    'FontSize',14,...
    'FitBoxToText','off');
% Now, for the estimation values
figure1 = figure;
axes1 = axes('Parent',figure1,'FontSize',14);
ylim(axes1,[0.905 1.02]);
xlim(axes1,[0.85, 7.15]);
box(axes1,'on');
hold(axes1,'on');
plot(T, estim(:,1),'MarkerSize',30,'Marker','.',...
    'Color',[0.87058824300766 0.490196079015732 0],'LineWidth',2,'Parent',axes1, 'DisplayName','TProd Upper Bound');
plot(T, n.^(-1./(2*T)).*estim(:,1)','MarkerSize',30,'Marker','.','LineStyle','--',...
    'Color',[0.87058824300766 0.490196079015732 0],'LineWidth',2,'Parent',axes1,'DisplayName','TProd Lower Bound');
plot( T, estim(:,2),'MarkerSize',10,'Marker','o',...
    'Color',[0.466666668653488 0.674509823322296 0.18823529779911],'LineWidth',2,'Parent',axes1, 'DisplayName','MPath Upper Bound');
plot(T, n.^(-1./(2*T)).*estim(:,2)','MarkerSize',10,'Marker','o','LineStyle','--',...
    'Color',[0.466666668653488 0.674509823322296 0.18823529779911],'LineWidth',2,'Parent',axes1, 'DisplayName','MPath Lower Bound');
legend1 = legend(axes1,'show');
set(legend1,...
    'Position',[0.151190493190806 0.196031616826646 0.376785706036856 0.226984120739831], ...
  'EdgeColor',[1 1 1],...
    'FontSize', 12);
annotation(figure1,'textbox',...
    [0.844642857142857 0.137095239301523 0.051785713274564 0.0642857130794298],...
    'String',{'T'},  ...
    'LineStyle','none',...
    'FontSize',14,...
    'FitBoxToText','off');
annotation(figure1,'textbox',...
    [0.141071428571429 0.856142858349143 0.0982142833194563 0.0642857130794299],...
    'String',{'Value'}, ...
    'LineStyle','none',...
    'FontSize',14,...
    'FitBoxToText','off');

%% -------- SECTION 4 - Exact computation of the CJSR ---------------------
% So now, let us check wether Theorem 3.8 is satisfied for a system.
% For the sake of the demonstration, 
%% we begin by checking wether it works for the original system (it wont).
% The process we follow is quite natural, given the statement of Theorem
% 3.8
% First, compute a very good multinorm. 
% We use the same code as before.
[estimateS, Q] = S.cjsrQuadMultinorm(opts);
% Qis an output of the algorithm, it is the quadratic multinorm
% with value estimateS, represented by its quadratic forms.

% Now, we seek to compute the set of edged E' such that for (a,b,s) \in E',
% min(abs(eigs(A_s' Q_b A_s - estimateS^2 Q_a))) = 0.
% In pratice, due to numerical issues, we use a slightly different
% approach:
gamma = zeros(S.getNEdges(), 1); % We will use this for computing E'.
for e = 1 : S.getNEdges()
    edge = S.getEdges(e) ; % Here, the edge is represented as (src, dst). The corresponding label is stored in S.getLabel(e)
    Qi = real(Q{edge(1)}) ; % To make sure there are no numerical errors
    Qj = real(Q{edge(2)}) ;
    % Now, things may seem wierd. We needed to take into account the fact
    % that an edge may carry a concatenation of labels (T-Product lift). 
    % Therefore, we needed to be able to access the matrix products
    % corresponding to these concatenated labels on the edges.
    As = S.getMatProdEdge(e) ; % We get the matrix product corresponding to the label (here, it is simply A_s for the edge (i,j,s))
    % And here's perhaps the wierdest part.
    % In order to be able effectively take into account the bisection
    % tolerance, we to not check wether min(abs(eigs(As'*Qj* As - estimateS^{2*(size of the label)}Qi))) = 0.
    % Instead, we solve the problem :
    %   min_\gamma \gamma : (As'*Qj*As -
    %          \gamma^{2*(size of the label)}Qi) <= 0;
    % This is a generalized eigenvalue problem, which matlab solves easily.
    % if \gamma ~~ estimateS, then we have a good edge!
    gamma(e) = eigs(Qi\(As'*Qj*As),1)^(1/2/S.getNLabels(e)) ;        
end
% we reject any edge such that $\gamma <= estimateS - tol$
% Indeed, from the bisection procedure which has a tolerance tol, we know that estimateS-tol is
% a value for which no quadratic multinorms exists.
goodEdges = find((estimateS - gamma) <= tol);
edgesPrime = S.getEdges(goodEdges);
% for checking whether we have a simple cycle or not, we do a simple trick:
% each node must appear once as source, and once at destination. (Multiple
% simple cycles may exist, but this is unlikely, and theorem 3.8 would
% still work as long as these cycles are distinct).
visitedNodes = unique(edgesPrime(:));
occurences = zeros(length(visitedNodes),1);
for v = 1 : length(visitedNodes)
    occurences(v) = occurences(v) + sum(edgesPrime(:) == visitedNodes(v));
end
% Now for the final check:
if occurences == 2 
    % We are done!
    % We reconstruct the (at least "a") simple cycle
    cycle = goodEdges(1);
    src = edgesPrime(1,1);
    curDst = edgesPrime(1,2);
    k = 1;
    while curDst ~= src
        nextEdgeIndex = find(edgesPrimes(:,1) == curDst);
        cycle = [cycle; nextEdgeIndex];
        edge = S.getEdge(nextEdgeIndex);
        curDst = edge(2);
        k = k + 1;
        if k >= lenght(goodEdges) + 1
            error('something went wrong.')
        end
    end
else
    % We did not get a cycle...
    % Here is the set of edges:
    disp('We could not manage to find a simple cycle. Here are the edges:')
    edgesPrime
    % and the culprit nodes are
    disp('The following nodes break the simplity of the cycle, or the cycle itself:')
    visitedNodes(occurences <= 1 | occurences >= 3)
end

%% We now apply the same technique to the M-pathdependant lift, M = 5.
SToTest = SMPDepCell{3}; % POINTER : you may plug in another system.
% We do the same as the above, replacing S by SToTest
[estimateS, Q] = SToTest.cjsrQuadMultinorm(opts);
% Qis an output of the algorithm, it is the quadratic multinorm
% with value estimateS, represented by its quadratic forms.

% Now, we seek to compute the set of edged E' such that for (a,b,s) \in E',
% min(abs(eigs(A_s' Q_b A_s - estimateS^2 Q_a))) = 0.
% In pratice, due to numerical issues, we use a slightly different
% approach:
%
gamma = zeros(SToTest.getNEdges(), 1); % We will use this for computing E'.
for e = 1 : SToTest.getNEdges()
    edge = SToTest.getEdges(e);  % Here, the edge is represented as (src, dst). The corresponding label is stored in S.getLabel(e)
    Qi = real(Q{edge(1)}) ; % To make sure there are no numerical errors
    Qj = real(Q{edge(2)}) ;
    % Now, things may seem wierd. We needed to take into account the fact
    % that an edge may carry a concatenation of labels (T-Product lift). 
    % Therefore, we needed to be able to access the matrix products
    % corresponding to these concatenated labels on the edges.
    As = SToTest.getMatProdEdge(e); % We get the matrix product corresponding to the label (here, it is simply A_s for the edge (i,j,s))
    % And here's perhaps the wierdest part.
    % In order to be able effectively take into account the bisection
    % tolerance, we to not check wether min(abs(eigs(As'*Qj* As - estimateS^{2*(size of the label)}Qi))) = 0.
    % Instead, we solve the problem :
    %   min_\gamma \gamma : (As'*Qj*As -
    %          \gamma^{2*(size of the label)}Qi) <= 0;
    % This is a generalized eigenvalue problem, which matlab solves easily.
    % if \gamma ~~ estimateS, then we have a good edge!
    gamma(e) = eigs(Qi\(As'*Qj*As),1)^(1/2/SToTest.getNLabels(e)) ;       
end
% we reject any edge such that $\gamma <= estimateS - tol$
% Indeed, from the bisection procedure which has a tolerance tol, we know that estimateS-tol is
% a value for which no quadratic multinorms exists.
goodEdges = find((estimateS - gamma) <= tol);
edgesPrime = SToTest.getEdges(goodEdges);
% for checking whether we have a simple cycle or not, we do a simple trick:
% each node must appear once as source, and once at destination. (Multiple
% simple cycles may exist, but this is unlikely, and theorem 3.8 would
% still work as long as these cycles are distinct).
visitedNodes = unique(edgesPrime(:));
occurences = zeros(length(visitedNodes),1);
for v = 1 : length(visitedNodes)
    occurences(v) = occurences(v) + sum(edgesPrime(:) == visitedNodes(v));
end
% Now for the final check:
if occurences == 2 
    % We are done!
    % We reconstruct the (at least "a") simple cycle
    cycle = goodEdges(1);
    src = edgesPrime(1,1);
    curDst = edgesPrime(1,2);
    k = 1;
    while curDst ~= src
        nextEdgeIndex = goodEdges(edgesPrime(:,1) == curDst);
        cycle = [cycle; nextEdgeIndex];
        edge = SToTest.getEdges(nextEdgeIndex);
        curDst = edge(2);
        k = k + 1;
        if k >= length(goodEdges) + 1
            error('something went wrong.')
        end
    end
    disp('The sufficient condition of Theorem 3.8 is found to be satisfied.')
    disp('Here is the period of a sequence of reaching the CJSR')
    labelCycle = SToTest.getLabels(cycle)'
    disp('The associated matrix product is, ')
    productCycle = SToTest.getMatProdEdge(cycle)
    disp('The CJSR is given by')
    totalCycleLength = 00;
    for k = 1 : length(labelCycle)
        totalCycleLength = totalCycleLength + length(labelCycle{k});
    end
    cjsr = max(abs(eig(productCycle)))^(1/totalCycleLength)
else
    % We did not get a cycle...
    % Here is the set of edges:
    disp('We could not manage to find a simple cycle. Here are the edges:')
    edgesPrime
    % and the culprit nodes are
    disp('The following nodes break the simplity of the cycle, or the cycle itself:')
    visitedNodes(occurences <= 1 | occurences >= 3)
end








